﻿Imports Microsoft.VisualBasic
Imports System
Imports System.Collections
Imports System.Collections.Generic
Imports System.Configuration
Imports System.Data
Imports System.Data.Sql
Imports System.Data.SqlClient
Imports System.Linq
Imports System.Web
Imports System.Web.Security
Imports System.Web.UI
Imports System.Web.UI.HtmlControls
Imports System.Web.UI.WebControls
Imports System.Web.UI.WebControls.WebParts
Imports System.Xml.Linq

Public Class ClaimDAO
    Inherits DAO

    Public Function getClaimList() As List(Of ClaimDTO)
        Return getClaimList(Nothing)
    End Function

    ' fetch list of claims which have not yet been approved 
    Public Function getClaimList(ByVal whereClause As String) As List(Of ClaimDTO)
        If IsNothing(whereClause) Then
            whereClause = ""
        End If
        Dim foundClaims As List(Of ClaimDTO) = New List(Of ClaimDTO)
        Dim myConn As SqlConnection = New SqlConnection(connectionString)
        myConn.Open()
        Dim mySelectStatement As String = "SELECT * FROM claim " + whereClause
        Dim myComm As SqlCommand = New SqlCommand(mySelectStatement, myConn)
        Dim myDR As SqlDataReader
        myDR = myComm.ExecuteReader()
        While myDR.Read()
            foundClaims.Add(makeDTO(myDR))
            ' create new DTO and add to list 
        End While
        myConn.Close()
        Return foundClaims
    End Function

    ' fetch a claim by ID or key
    Public Function getClaim(ByVal claimKey As Integer) As ClaimDTO
        Dim myConn As SqlConnection = New SqlConnection(connectionString)
        myConn.Open()
        Dim mySelectStatement As String = "SELECT * FROM claim WHERE (claimID=" _
            + claimKey.ToString() + ")"
        Dim myComm As SqlCommand = New SqlCommand(mySelectStatement, myConn)
        Dim myDR As SqlDataReader
        myDR = myComm.ExecuteReader()
        myConn.Close()
        If myDR.Read() = False Then
            Return Nothing
        End If
        Return makeDTO(myDR)
    End Function

    ' fetch a claim by reference number (the first one to match) 
    Public Function getClaimByRef(ByVal claimRef As String) As ClaimDTO
        Dim myConn As SqlConnection = New SqlConnection(connectionString)
        myConn.Open()
        Dim mySelectStatement As String = "SELECT * FROM claim WHERE (reference='" _
            + claimRef + "')"
        Dim myComm As SqlCommand = New SqlCommand(mySelectStatement, myConn)
        Dim myDr As SqlDataReader = myComm.ExecuteReader()
        If myDr.Read() = False Then
            Return Nothing
        End If
        Dim myClaim As ClaimDTO = makeDTO(myDr)
        myConn.Close()
        Return myClaim
    End Function

    ' helper method 
    Private Function makeDTO(ByVal myDR As SqlDataReader) As ClaimDTO
        Dim myClaim As ClaimDTO = New ClaimDTO()
        myClaim.claimID = myDR.GetInt32(0)
        myClaim.reference = myDR.GetString(1)
        myClaim.description = myDR.GetString(2)
        myClaim.amount = myDR.GetDecimal(3)
        myClaim.isApproved = myDR.GetString(4).StartsWith("Y")
        myClaim.claimDate = myDR.GetDateTime(5)
        myClaim.policyID = myDR.GetInt32(6)
        Return myClaim
    End Function

    ' save a claim object to the database 
    Public Sub saveClaim(ByVal myClaim As ClaimDTO)
        Dim isApproved As Char = "N"
        If myClaim.isApproved Then
            isApproved = "Y"
        End If
        Dim claimDate As String = myClaim.claimDate.Day.ToString() + "/" + myClaim.claimDate.Month.ToString() + "/" + myClaim.claimDate.Year.ToString()
        Dim myConn As SqlConnection = New SqlConnection(connectionString)
        myConn.Open()
        Dim insertString As String = "INSERT INTO claim (reference, description, amount, approved, claimDate, policyID) " + _
            "VALUES(@reference, @description, @amount, @approved, @claimDate, @policyID)"
        Dim myComm As SqlCommand = New SqlCommand(insertString, myConn)

        myComm.Parameters.AddWithValue("@reference", myClaim.reference)
        myComm.Parameters.AddWithValue("@description", myClaim.description)
        myComm.Parameters.AddWithValue("@amount", myClaim.amount)
        myComm.Parameters.AddWithValue("@approved", isApproved)
        myComm.Parameters.AddWithValue("@claimDate", myClaim.claimDate)
        myComm.Parameters.AddWithValue("@policyID", myClaim.policyID)

        Dim rows As Integer = myComm.ExecuteNonQuery()
        myConn.Close()
        If rows <> 1 Then
            Throw New Exception("Unexpected Result for Insert")
        End If
    End Sub

    Public Sub linkToPolicy(ByVal claim As ClaimDTO, ByVal myPolicy As PolicyDTO)
        Dim reference As String = claim.reference
        Dim policyNumber As String = myPolicy.policyNumber.ToString()
        Dim myConn As SqlConnection = New SqlConnection(connectionString)
        myConn.Open()
        Dim mySelectStatement As String = "SELECT policyID FROM policy WHERE (policyNumber=" _
            + policyNumber.ToString() + ")"
        Dim myComm As SqlCommand = New SqlCommand(mySelectStatement, myConn)
        Dim myDR As SqlDataReader = myComm.ExecuteReader()
        If myDR.Read() = False Then
            Return
        End If
        myPolicy.policyID = myDR.GetInt32(0)
        myConn.Close()
        myConn.Open()
        myComm = New SqlCommand("UPDATE claim SET policyID = @policyID WHERE reference = @reference", _
            myConn)
        myComm.Parameters.AddWithValue("@policyID", myPolicy.policyID)
        myComm.Parameters.AddWithValue("@reference", claim.reference)
        Dim rows As Integer = myComm.ExecuteNonQuery()
        myConn.Close()
        myConn.Close()
        If rows <> 1 Then
            Throw New Exception("Unexpected Result for Update")
        End If
    End Sub

    ' update claim object in the database 
    Public Sub updateClaim(ByVal myClaim As ClaimDTO)
        Dim myConn As SqlConnection = New SqlConnection(connectionString)
        myConn.Open()
        Dim myComm As SqlCommand = New SqlCommand("UPDATE claim SET reference = @reference, description = @description, amount = @amount, approved = @approved, claimDate = @claimDate, policyID = @policyID WHERE claimID = @claimID", _
            myConn)
        myComm.Parameters.AddWithValue("@claimID", myClaim.claimID)
        myComm.Parameters.AddWithValue("@reference", myClaim.reference)
        myComm.Parameters.AddWithValue("@description", myClaim.description)
        myComm.Parameters.AddWithValue("@amount", myClaim.amount)
        If myClaim.isApproved Then
            myComm.Parameters.AddWithValue("@approved", "Y")
        Else
            myComm.Parameters.AddWithValue("@approved", "N")
        End If
        myComm.Parameters.AddWithValue("@claimDate", myClaim.claimDate)
        myComm.Parameters.AddWithValue("@policyID", myClaim.policyID)
        Dim rows As Integer = myComm.ExecuteNonQuery()
        myConn.Close()
        If rows <> 1 Then
            Throw New Exception("Unexpected Result for Update")
        End If
    End Sub

    ' delete claim object from the database 
    Public Sub deleteClaim(ByVal myClaim As ClaimDTO)
        Dim myConn As SqlConnection = New SqlConnection(connectionString)
        myConn.Open()
        Dim myComm As SqlCommand = New SqlCommand("DELETE FROM claim WHERE claimID = @claimID", _
            myConn)
        myComm.Parameters.AddWithValue("@claimID", myClaim.claimID)
        Dim rows As Integer = myComm.ExecuteNonQuery()
        myConn.Close()
        If rows <> 1 Then
            Throw New Exception("Unexpected Result for Delete")
        End If
    End Sub


End Class
